package com.microsoft.notes

import com.microsoft.notes.appstore.AppState
import com.microsoft.notes.appstore.AppStore
import com.microsoft.notes.appstore.Navigation
import com.microsoft.notes.appstore.PreferenceKeys
import com.microsoft.notes.appstore.action.CompoundAction
import com.microsoft.notes.appstore.action.NavigationAction
import com.microsoft.notes.appstore.action.NavigationAction.HideArchivedNotesAction
import com.microsoft.notes.appstore.action.NavigationAction.NavigateToAction
import com.microsoft.notes.appstore.action.NavigationAction.ResetNavigationAction
import com.microsoft.notes.appstore.action.NavigationAction.ShowArchivedNotesAction
import com.microsoft.notes.appstore.action.NoteUIAction.FetchNoteDetailsAction
import com.microsoft.notes.appstore.action.ReadAction.LoadPreferencesAction
import com.microsoft.notes.components.UiMainActivityComponent
import com.microsoft.notes.controllerview.ControllerView
import com.microsoft.notes.noteslib.NotesLibrary
import com.microsoft.notes.richtext.scheme.Document
import com.microsoft.notes.richtext.scheme.Media
import com.microsoft.notes.store.Note
import com.microsoft.notes.utils.threading.ThreadExecutor

class MainActivityControllerView(
        val uiActivityComponent: UiMainActivityComponent,
        appStore: AppStore = com.microsoft.notes.appStore,
        mainThread: ThreadExecutor? = uiThread) : ControllerView(appStore, mainThread) {

    private var madeAuthRequest = false

    var currentDisplayedNav: Navigation = appState.navigation.lastOrNull() ?: Navigation.NOTES_LIST
        private set

    val previousStateNav: Navigation?
        get() = with(appState.navigation) { if (size > 1) this[size - 2] else null }

    override fun handleState(appState: AppState) {
        val authMethod = appState.preferences[PreferenceKeys.AUTH_METHOD]
        if (authMethod != null && appState.auth == null && !madeAuthRequest) {
            madeAuthRequest = true
            uiActivityComponent.authenticate(authMethod)
        }
        if (appState.navigation.isNotEmpty() && appState.navigation.last() != currentDisplayedNav) {
            uiActivityComponent.navigateTo(appState.navigation.last())
            currentDisplayedNav = appState.navigation.last()
        }
    }

    fun appLaunchedToDefaultView() {
        currentDisplayedNav = uiActivityComponent.defaultView
        NotesLibrary.getInstance().fetchNotes().then {
            appStore.dispatch(CompoundAction(
                    LoadPreferencesAction(),
                    ResetNavigationAction(listOf(uiActivityComponent.defaultView))))
        }
    }

    fun appLaunchedToEmptyNote() {
        currentDisplayedNav = Navigation.EDIT_NOTE
        val newNote = Note()
        NotesLibrary.getInstance().fetchNotes().then {
            NotesLibrary.getInstance().addNote(newNote).then {
                appStore.dispatch(CompoundAction(
                        LoadPreferencesAction(),
                        FetchNoteDetailsAction(newNote.localId),
                        ResetNavigationAction(listOf(uiActivityComponent.defaultView, Navigation.EDIT_NOTE))
                ))
            }
        }
    }

    fun appLaunchedToEditNote(noteId: String) {
        currentDisplayedNav = Navigation.EDIT_NOTE
        NotesLibrary.getInstance().fetchNotes().then {
            appStore.dispatch(CompoundAction(
                    LoadPreferencesAction(),
                    FetchNoteDetailsAction(noteId),
                    ResetNavigationAction(listOf(uiActivityComponent.defaultView, Navigation.EDIT_NOTE))
            ))
        }
    }

    fun createMediaNote(uri: String) {
        val document = Document(blocks = listOf(Media(localUrl = uri)))
        val note = Note(document = document)
        NotesLibrary.getInstance().addNote(note)
    }

    fun resetNavigationToNotesList() {
        appStore.dispatch(ResetNavigationAction(listOf(Navigation.NOTES_LIST)))
    }

    fun navigatingToSearch() {
        appStore.dispatch(NavigateToAction(Navigation.SEARCH))
    }

    fun navigatingBack() = appStore.dispatch(NavigationAction.NavigateBackAction())
}