package com.microsoft.notes

import android.os.Bundle
import android.support.v4.app.Fragment
import android.view.View
import android.view.ViewTreeObserver
import com.microsoft.notes.controllerview.ControllerView

abstract class ViewFragment<T : ControllerView> : Fragment() {

    //TODO Use lateinit here when we get kotlin 1.2, will save us the null checks
    protected var cv: T? = null
    private val firstLayoutListeners = mutableListOf<(View) -> Unit>()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        view.viewTreeObserver.addOnGlobalLayoutListener(object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                firstLayoutListeners.forEach { it(this@ViewFragment.view!!) }
                firstLayoutListeners.clear()
                view.viewTreeObserver.removeOnGlobalLayoutListener(this)
            }
        })
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)

        cv = setupControllerView()
    }

    override fun onStart() {
        super.onStart()
        cv?.onStart()
    }

    override fun onPause() {
        super.onPause()
        cv?.onPause()
    }

    override fun onResume() {
        super.onResume()
        cv?.onResume()
    }

    override fun onStop() {
        super.onStop()
        cv?.onStop()
    }

    override fun onDestroy() {
        super.onDestroy()
        cv?.onDestroy()
    }

    abstract fun setupControllerView(): T

    fun addFirstLayoutListener(listener: (View) -> Unit) {
        firstLayoutListeners.add(listener)
    }
}