package com.microsoft.notes.appstore

import com.microsoft.notes.store.Color
import com.microsoft.notes.store.Note
import com.microsoft.notes.store.extensions.describe

enum class Navigation {
    NOTES_LIST,
    SEARCH,
    EDIT_NOTE,
    PHOTO_VIEW,
    PREFERENCES,
    DEV_PREFS
}

object PreferenceKeys {
    const val AUTH_METHOD = "auth_method"
}

object AuthMethods {
    const val AAD = "AAD"
}

sealed class ScrollTo {
    object Top : ScrollTo()
    object Bottom : ScrollTo()
    class Custom(val position: Int) : ScrollTo()
    object NoScroll : ScrollTo()
}

data class NotesList(
        val notesCollection: List<Note> = emptyList(),
        val archivedNotesVisible: Boolean = false,
        val filteredColor: Color = Color.ORANGE,
        val scrollTo: ScrollTo = ScrollTo.NoScroll) {

    fun getNote(id: String): Note? {
        return notesCollection.find { it.localId == id }
    }

    fun replaceNote(replacement: Note): NotesList {
        val newCollection = notesCollection.map {
            if (it.localId == replacement.localId) {
                replacement
            } else {
                it
            }
        }
        return copy(notesCollection = newCollection)
    }

    fun addNotes(notes: List<Note>): NotesList {
        return copy(notesCollection = this.notesCollection + notes)
    }

    fun replaceNotes(notes: List<Note>): NotesList {
        val newCollection = notes.fold(this.notesCollection) { notesCollection, replacement ->
            notesCollection.map {
                if (it.localId == replacement.localId) {
                    replacement
                } else {
                    it
                }
            }
        }
        return copy(notesCollection = newCollection)
    }

    fun deleteNotes(notes: List<Note>): NotesList {
        val newCollection = this.notesCollection.filter {
            !notes.contains(it)
        }
        return copy(notesCollection = newCollection)
    }

}

data class EditNote(val currentNoteId: String = "")

data class Auth(val email: String)

data class AppState(
        val notesList: NotesList = NotesList(),
        val editNote: EditNote = EditNote(),
        val navigation: List<Navigation> = emptyList(),
        val auth: Auth? = null,
        val preferences: Map<String, String> = emptyMap()) {
    override fun toString(): String {
        val stringBldr = StringBuilder()
        val DELIMITER = "\n: "
        stringBldr.append(notesList.notesCollection.describe())
                .append(DELIMITER + editNote)
                .append(DELIMITER + navigation)
                .append(DELIMITER + auth)
                .append(DELIMITER + preferences)
        return stringBldr.toString()
    }
}

fun AppState.getCurrentEditNote(): Note? =
        notesList.getNote(editNote.currentNoteId)