package com.microsoft.notes.appstore.reducer

import com.microsoft.notes.appstore.Auth
import com.microsoft.notes.appstore.AuthMethods
import com.microsoft.notes.appstore.PreferenceKeys
import com.microsoft.notes.appstore.AppState
import com.microsoft.notes.appstore.action.AuthAction
import com.microsoft.notes.appstore.action.AuthAction.AADLoginAction
import com.microsoft.notes.appstore.action.AuthAction.LogOutAction

object AuthReducer {

    fun reduce(action: AuthAction, currentAppState: AppState): AppState =
            currentAppState.copy(auth = reduceAuth(action),
                    preferences = reducePreferences(action, currentAppState.preferences))

    fun reduceAuth(action: AuthAction): Auth? = when (action) {
        is AADLoginAction -> Auth(action.email)
        is LogOutAction -> null
    }

    fun reducePreferences(action: AuthAction, preferences: Map<String, String>): Map<String, String> =
            when (action) {
                is AADLoginAction -> preferences + Pair(PreferenceKeys.AUTH_METHOD, AuthMethods.AAD)
                is LogOutAction -> preferences - PreferenceKeys.AUTH_METHOD
            }
}
