package com.microsoft.notes.appstore.reducer

import com.microsoft.notes.appstore.Navigation
import com.microsoft.notes.appstore.NotesList
import com.microsoft.notes.appstore.AppState
import com.microsoft.notes.appstore.action.NavigationAction
import com.microsoft.notes.appstore.action.NavigationAction.ChangeColorFilterAction
import com.microsoft.notes.appstore.action.NavigationAction.HideArchivedNotesAction
import com.microsoft.notes.appstore.action.NavigationAction.NavigateBackAction
import com.microsoft.notes.appstore.action.NavigationAction.NavigateToAction
import com.microsoft.notes.appstore.action.NavigationAction.ResetNavigationAction
import com.microsoft.notes.appstore.action.NavigationAction.ShowArchivedNotesAction
import com.microsoft.notes.store.Color

object NavigationReducer {

    fun reduce(action: NavigationAction, currentAppState: AppState): AppState =
            when (action) {
                is NavigateToAction, is NavigateBackAction, is ResetNavigationAction ->
                    currentAppState.copy(navigation = reduceNavigation(
                            action, currentAppState.navigation))
                else -> currentAppState.copy(notesList = reduceNotesList(
                        action, currentAppState.notesList))
            }

    fun reduceNotesList(action: NavigationAction, notesList: NotesList) =
            notesList.copy(archivedNotesVisible = reduceArchiveNotesVisible(
                    action, notesList.archivedNotesVisible),
                    filteredColor = reduceFilteredColor(
                            action, notesList.filteredColor))

    private fun reduceArchiveNotesVisible(action: NavigationAction, visible: Boolean): Boolean =
            when (action) {
                is ShowArchivedNotesAction -> true
                is HideArchivedNotesAction -> false
                else -> visible
            }

    private fun reduceFilteredColor(action: NavigationAction, filteredColor: Color) =
            when (action) {
                is ChangeColorFilterAction -> action.color
                else -> filteredColor
            }

    fun reduceNavigation(action: NavigationAction, currentNavigation: List<Navigation>) =
            when (action) {
                is NavigateToAction -> currentNavigation + action.navigation
                is NavigateBackAction -> currentNavigation.dropLast(1)
                is ResetNavigationAction -> action.navigation
                else -> currentNavigation
            }
}
