package com.microsoft.notes.appstore.reducer

import com.microsoft.notes.appstore.EditNote
import com.microsoft.notes.appstore.NotesList
import com.microsoft.notes.appstore.ScrollTo
import com.microsoft.notes.appstore.AppState
import com.microsoft.notes.appstore.action.NoteUIAction
import com.microsoft.notes.appstore.action.NoteUIAction.ChangeNoteBackgroundAction
import com.microsoft.notes.appstore.action.NoteUIAction.FetchNoteDetailsAction
import com.microsoft.notes.store.Note
import com.microsoft.notes.store.extensions.findAndMap

object NoteUIReducer {

    fun reduce(action: NoteUIAction, currentAppState: AppState): AppState =
            with(currentAppState) {
                return currentAppState.copy(
                        editNote = reduceEditNote(action, editNote),
                        notesList = reduceNotesList(action, notesList)
                )
            }

    private fun reduceEditNote(action: NoteUIAction, editNote: EditNote): EditNote =
            when (action) {
                is FetchNoteDetailsAction -> editNote.copy(currentNoteId = action.localId)
                else -> editNote
            }

    private fun reduceNotesList(action: NoteUIAction, notesList: NotesList): NotesList {
        return notesList.copy(notesCollection = reduceNotesCollection(action, notesList.notesCollection),
                scrollTo = ScrollTo.NoScroll)
    }

    fun reduceNotesCollection(action: NoteUIAction, currentNotesCollection: List<Note>): List<Note> =
            when (action) {
                is NoteUIAction.NotesCollectionUpdatedAction -> action.notesCollection
                is ChangeNoteBackgroundAction ->
                    currentNotesCollection.findAndMap(
                            find = { it.localId == action.localId }, map = { it.copy(color = action.color) })
                else -> currentNotesCollection
            }
}
