package com.microsoft.notes.auth

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Build
import android.util.Log
import android.webkit.CookieManager
import android.webkit.CookieSyncManager
import com.microsoft.notes.appstore.AppStore
import com.microsoft.notes.appstore.action.AuthAction
import com.microsoft.notes.noteslib.NotesLibrary
import com.microsoft.notes.authentication.Authenticator;
import com.microsoft.notes.authentication.AuthResultListener;
import com.microsoft.notes.authentication.AuthResult;

class AADAuthProvider(val context: Context, val appStore: AppStore) : AuthProvider {

    private val TAG = this::class.java.simpleName

    var authenticator = Authenticator(context);

    override fun login(activity: Activity, onSuccess: (() -> Unit)?, onError: ((AuthFailedReason) -> Unit)?) {

        val authCallback = object : AuthResultListener {
            override fun onError(error: String?) {
                val reason = AuthFailedReason.OTHER
                onError?.invoke(reason)
            }
            override fun onSuccess(result: AuthResult?) {
                if (result == null || result.token == null || result.token.isEmpty()) {
                    Log.d(TAG, "Token is empty!")
                    onError?.invoke(AuthFailedReason.NO_TOKEN)
                } else {
                    Log.d(TAG, "Token is available")
                    with(result) {
                        NotesLibrary.getInstance().newAuthToken(token).then {
                            appStore.dispatch(AuthAction.AADLoginAction(email = emailID))
                            onSuccess?.invoke()
                        }
                    }
                }
            }
        }

        authenticator.login(activity, authCallback)
    }

    override fun logout() {
        cleanCookies()
        NotesLibrary.getInstance().logout().then {
            NotesLibrary.getInstance().deleteAllNotes()
            appStore.dispatch(AuthAction.LogOutAction())
        }
    }

    override fun onResultAfterRequestToken(requestCode: Int, resultCode: Int, data: Intent?) {
        authenticator.onResultAfterRequestToken(requestCode, resultCode, data)
    }

    fun cleanCookies() {
        val cookieManager = CookieManager.getInstance()

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            cookieManager.removeAllCookies(null)
            cookieManager.flush()
        } else {
            cookieManager.removeAllCookie()
            CookieSyncManager.getInstance().sync()
        }
    }
}