package com.microsoft.notes.components

import android.content.Context
import android.content.Intent
import android.support.v7.app.AppCompatActivity
import com.microsoft.notes.ActivityStateManager
import com.microsoft.notes.MainActivityControllerView
import com.microsoft.notes.controllerview.ControllerView
import com.microsoft.notes.flavor.FlavorConfiguration
import rx.subjects.PublishSubject

interface ViewActivityComponent {
    val flavorConfiguration: FlavorConfiguration

    val showNotesActionBar: Boolean

    val subActivityResultSubscribers: SubActivityResultSubscribers

    val activity: AppCompatActivity

    fun registerForCrashes()

    fun registerForUpdates()

    fun unregisterForUpdates()

    fun registerForFeedback(context: Context)

    fun showFeedbackActivity(context: Context)

    fun canHandleDevOptions(): Boolean

    fun canHandleHockeyApp(): Boolean

    fun isUnderTest(): Boolean

    fun startPolling()

    fun stopPolling()

    fun onFragmentLoadComplete(fragmentId: Int)
}

interface ViewActivityWithControllerComponent<T : ControllerView> : ViewActivityComponent {
    val cv: T

    fun setupControllerView(): T
}

interface ViewActivityWithStateManagerComponent : ViewActivityWithControllerComponent<MainActivityControllerView> {
    val stateManager: ActivityStateManager

    fun viewPhotoInFullScreen(mediaId: String?)
}

class SubActivityResultSubscribers {
    val permissionRequestResults: PublishSubject<RequestPermissionsResult> = PublishSubject.create()
    val activityResults: PublishSubject<ActivityResult> = PublishSubject.create()
}

class ActivityResult(val requestCode: Int, val resultCode: Int, val intent: Intent?)

class RequestPermissionsResult(val requestCode: Int, val permissions: Array<out String>, val grantResults: IntArray)
