package com.microsoft.notes.controllerview

import com.microsoft.notes.appstore.AppState
import com.microsoft.notes.appstore.StateHandler
import com.microsoft.notes.appstore.AppStore
import com.microsoft.notes.appstore.getCurrentEditNote
import com.microsoft.notes.noteslib.NotesLibrary
import com.microsoft.notes.store.Note
import com.microsoft.notes.utils.logging.Categories
import com.microsoft.notes.utils.logging.EventMarkers
import com.microsoft.notes.utils.logging.SeverityLevel
import com.microsoft.notes.utils.threading.ThreadExecutor

abstract class ControllerView(val appStore: AppStore,
                              val mainThread: ThreadExecutor? = null)
    : LifecycleCallbacks, StateHandler(mainThread) {

    override var isActivityRunning: Boolean = false

    val appState: AppState
        get() = appStore.state

    val currentNote: Note?
        get() = appState.getCurrentEditNote()

    override fun onStart() {
        isActivityRunning = true
        appStore.stateHandlers.add(this)
        handle(appStore.state)
    }

    override fun onResume() {
    }

    override fun onPause() {
    }

    override fun onStop() {
        isActivityRunning = false
    }

    override fun onDestroy() {
        appStore.stateHandlers.remove(this)
    }

    fun finish() {
        isActivityRunning = false
    }

    fun recordTelemetry(eventMarker: EventMarkers, vararg keyValuePairs: Pair<String, String>,
                        severityLevel: SeverityLevel = SeverityLevel.Info,
                        category: Categories = Categories.NotesSDKApp) {
        NotesLibrary.getInstance().recordTelemetry(eventMarker = eventMarker, keyValuePairs = *keyValuePairs,
                severityLevel = severityLevel, category = category)
    }

    override fun handle(data: AppState) {
        if (isActivityRunning) handleState(appState)
    }

    abstract fun handleState(appState: AppState)
}