package com.microsoft.notes.editnote

import com.microsoft.notes.appstore.Navigation
import com.microsoft.notes.appstore.AppState
import com.microsoft.notes.appstore.AppStore
import com.microsoft.notes.appstore.action.NavigationAction.NavigateBackAction
import com.microsoft.notes.appstore.action.NavigationAction.NavigateToAction
import com.microsoft.notes.appstore.getCurrentEditNote
import com.microsoft.notes.components.UiEditNotesFragmentComponent
import com.microsoft.notes.controllerview.ControllerView
import com.microsoft.notes.noteslib.NotesLibrary
import com.microsoft.notes.richtext.scheme.Document
import com.microsoft.notes.store.Note
import com.microsoft.notes.uiThread
import com.microsoft.notes.utils.threading.ThreadExecutor

class EditNoteControllerView(
        val uiFragmentComponent: UiEditNotesFragmentComponent,
        appStore: AppStore = com.microsoft.notes.appStore,
        mainThread: ThreadExecutor? = uiThread) : ControllerView(appStore, mainThread) {

    fun updateNoteWithTitle(titleText: String) {
        with(appState) {
            updateNoteWithTitle(noteLocalId = editNote.currentNoteId, titleText = titleText)
        }
    }

    fun updateNoteWithDocument(document: Document) {
        with(appState) {
            updateNoteWithDocument(noteLocalId = editNote.currentNoteId,
                    newDocument = document,
                    oldDocument = currentNote?.document)
        }
    }

    private fun updateNoteWithTitle(noteLocalId: String,
                                    titleText: String) {
        NotesLibrary.getInstance().updateNoteTitle(noteLocalId, titleText)
    }

    private fun updateNoteWithDocument(noteLocalId: String,
                                       newDocument: Document,
                                       oldDocument: Document?) {
        NotesLibrary.getInstance().updateNoteDocument(noteLocalId, newDocument, oldDocument)
    }

    fun deleteMediaFromNote(mediaLocalId: String) {
        NotesLibrary.getInstance().deleteMedia(mediaLocalId)
    }

    fun viewPhotoInFullScreen() {
        appStore.dispatch(NavigateToAction(Navigation.PHOTO_VIEW))
    }

    fun navigatingBack() {
        appStore.dispatch(NavigateBackAction())
    }

    override fun handleState(appState: AppState) {
        appState.getCurrentEditNote()?.let { setNoteDetails(it) }
    }

    fun setNoteDetails(note: Note) {
        uiFragmentComponent.setNoteDetails(note)
    }
}