package com.microsoft.notes.extensions

import android.app.Activity
import android.content.ContentResolver
import android.content.Intent
import android.net.Uri
import android.provider.MediaStore
import android.provider.Settings
import android.support.v7.app.AlertDialog

fun Activity.resIdToUri(resId: Int): Uri =
        Uri.Builder()
                .scheme(ContentResolver.SCHEME_ANDROID_RESOURCE)
                .authority(this.resources.getResourcePackageName(resId))
                .appendPath(this.resources.getResourceTypeName(resId))
                .appendPath(this.resources.getResourceEntryName(resId))
                .build()

fun Activity.resIdToString(resId: Int): String = resIdToUri(resId).toString()

fun Activity.launchSystemSettingsActivityForApp() {
    val intent = Intent()
    intent.action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
    val uri = Uri.fromParts("package", packageName, null)
    intent.data = uri
    startActivity(intent)
}

fun Activity.addMediaToGallery(uri: Uri) {
    val mediaScanIntent = Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE)
    mediaScanIntent.data = uri
    this.sendBroadcast(mediaScanIntent)
}

fun Activity.alert(title: String,
                   positiveButtonText: String,
                   positiveButtonFn: () -> Unit,
                   negativeButtonText: String,
                   negativeButtonFn: () -> Unit = {}): AlertDialog =
        AlertDialog.Builder(this)
                .setTitle(title)
                .setPositiveButton(positiveButtonText) { _, _ -> positiveButtonFn() }
                .setNegativeButton(negativeButtonText) { _, _ -> negativeButtonFn() }
                .create()