package com.microsoft.notes.extensions

import android.animation.Animator
import android.animation.AnimatorListenerAdapter
import android.animation.AnimatorSet
import android.animation.TimeInterpolator
import android.animation.ValueAnimator
import android.support.v4.view.animation.FastOutLinearInInterpolator
import android.support.v4.view.animation.FastOutSlowInInterpolator
import android.support.v4.view.animation.LinearOutSlowInInterpolator

fun List<Animator>.animateTogether(): AnimatorSet {
    val animatorSet = AnimatorSet()
    animatorSet.playTogether(this)
    return animatorSet
}

fun animateTogether(vararg animators: Animator): AnimatorSet {
    val animatorSet = AnimatorSet()
    animatorSet.playTogether(animators.asList())
    return animatorSet
}

fun List<Animator>.animateSequentially(): AnimatorSet {
    val animatorSet = AnimatorSet()
    animatorSet.playSequentially(this)
    return animatorSet
}

fun animateSequentially(vararg animators: Animator): AnimatorSet {
    val animatorSet = AnimatorSet()
    animatorSet.playSequentially(animators.asList())
    return animatorSet
}

fun ValueAnimator.withUpdateListener(listener: (ValueAnimator) -> Unit): ValueAnimator {
    addUpdateListener(listener)
    return this
}

fun Animator.onStart(listener: (Animator) -> Unit): Animator {
    addListener(object : AnimatorListenerAdapter() {
        override fun onAnimationStart(animation: Animator?) {
            listener(this@onStart)
        }
    })
    return this
}

fun Animator.onEnd(listener: (Animator) -> Unit): Animator {
    addListener(object : AnimatorListenerAdapter() {
        override fun onAnimationEnd(animation: Animator?) {
            listener(this@onEnd)
        }
    })
    return this
}

fun Animator.accelerate(): Animator {
    interpolator = FastOutLinearInInterpolator()
    return this
}

fun Animator.accelerateOrDecelerate(accelerate: Boolean): Animator {
    if (accelerate) accelerate() else decelerate()
    return this
}

fun Animator.accelerateAndDecelerate(): Animator {
    interpolator = FastOutSlowInInterpolator()
    return this
}

fun Animator.decelerate(): Animator {
    interpolator = LinearOutSlowInInterpolator()
    return this
}

fun Animator.withInterpolator(interpolator: TimeInterpolator): Animator {
    this.interpolator = interpolator
    return this
}

fun Animator.withStartDelay(delay: Long): Animator {
    this.startDelay = delay
    return this
}