package com.microsoft.notes.extensions

import android.content.Context
import android.graphics.drawable.ColorDrawable
import android.graphics.drawable.Drawable
import android.os.Build
import android.support.v4.content.ContextCompat
import android.util.TypedValue
import com.microsoft.notes.R
import com.microsoft.notes.store.Color
import com.microsoft.notes.store.Note

enum class RoundedSide { NONE, START, END }

fun Color.toColorAttr(): Int =
        when (this) {
            Color.ORANGE -> R.attr.themeColorNoteOrange
            Color.YELLOW -> R.attr.themeColorNoteYellow
            Color.GREEN -> R.attr.themeColorNoteGreen
            Color.PINK -> R.attr.themeColorNotePink
            Color.PURPLE -> R.attr.themeColorNotePurple
            Color.BLUE -> R.attr.themeColorNoteBlue
        }

fun Color.toOverlayColorResource(): Int =
        when (this) {
            Color.BLUE -> R.color.black
            else -> R.color.white
        }

fun Color.toRoundedStartDrawable(): Int =
        when (this) {
            Color.ORANGE -> R.drawable.note_color_orange_rounded_4dp_start
            Color.YELLOW -> R.drawable.note_color_yellow_rounded_4dp_start
            Color.GREEN -> R.drawable.note_color_green_rounded_4dp_start
            Color.PINK -> R.drawable.note_color_pink_rounded_4dp_start
            Color.PURPLE -> R.drawable.note_color_purple_rounded_4dp_start
            Color.BLUE -> R.drawable.note_color_blue_rounded_4dp_start
        }

fun Color.toRoundedEndDrawable(): Int =
        when (this) {
            Color.ORANGE -> R.drawable.note_color_orange_rounded_4dp_end
            Color.YELLOW -> R.drawable.note_color_yellow_rounded_4dp_end
            Color.GREEN -> R.drawable.note_color_green_rounded_4dp_end
            Color.PINK -> R.drawable.note_color_pink_rounded_4dp_end
            Color.PURPLE -> R.drawable.note_color_purple_rounded_4dp_end
            Color.BLUE -> R.drawable.note_color_blue_rounded_4dp_end
        }

fun Color.toRoundedDrawable(context: Context, roundedSide: RoundedSide = RoundedSide.NONE): Drawable =
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            ColorDrawable(toContextColor(context))
        } else {
            when (roundedSide) {
                RoundedSide.NONE -> ColorDrawable(toContextColor(context))
                RoundedSide.START -> ContextCompat.getDrawable(context, toRoundedStartDrawable())
                RoundedSide.END -> ContextCompat.getDrawable(context, toRoundedEndDrawable())
            }
        }

fun Color.toContextColor(context: Context): Int {
    val typedValue = TypedValue()
    context.theme.resolveAttribute(toColorAttr(), typedValue, true)
    return typedValue.data
}

fun Color.toOverlayContextColor(context: Context): Int =
        ContextCompat.getColor(context, this.toOverlayColorResource())

fun Note.getStableId(): Long = this.localId.hashCode().toLong()