package com.microsoft.notes.extensions

import android.os.Build
import android.support.annotation.RequiresApi
import android.support.v4.view.animation.FastOutLinearInInterpolator
import android.support.v4.view.animation.FastOutSlowInInterpolator
import android.support.v4.view.animation.LinearOutSlowInInterpolator
import android.transition.SidePropagation
import android.transition.Transition
import android.transition.TransitionSet
import android.view.View

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun Transition.addTargets(vararg ids: Int): Transition {
    ids.forEach { addTarget(it) }
    return this
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun Transition.addTargets(vararg names: String): Transition {
    names.forEach { addTarget(it) }
    return this
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun Transition.addTargets(vararg views: View): Transition {
    views.forEach { addTarget(it) }
    return this
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun Transition.accelerate(): Transition {
    interpolator = FastOutLinearInInterpolator()
    return this
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun Transition.accelerateOrDecelerate(accelerate: Boolean): Transition {
    if (accelerate) accelerate() else decelerate()
    return this
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun Transition.accelerateAndDecelerate(): Transition {
    interpolator = FastOutSlowInInterpolator()
    return this
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun Transition.decelerate(): Transition {
    interpolator = LinearOutSlowInInterpolator()
    return this
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun Transition.withSidePropagation(side: Int, speed: Float): Transition {
    val propagation = SidePropagation()
    propagation.setSide(side)
    propagation.setPropagationSpeed(speed)
    this.propagation = propagation
    return this
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
fun TransitionSet.addTransitions(vararg transitions: Transition): TransitionSet {
    transitions.forEach { addTransition(it) }
    return this
}