package com.microsoft.notes.noteslist

import com.microsoft.notes.appstore.Navigation
import com.microsoft.notes.appstore.AppState
import com.microsoft.notes.appstore.AppStore
import com.microsoft.notes.appstore.action.CompoundAction
import com.microsoft.notes.appstore.action.NavigationAction.NavigateToAction
import com.microsoft.notes.appstore.action.NoteUIAction.FetchNoteDetailsAction
import com.microsoft.notes.components.UiNotesFragmentComponent
import com.microsoft.notes.controllerview.ControllerView
import com.microsoft.notes.noteslib.NotesLibrary
import com.microsoft.notes.richtext.scheme.Content
import com.microsoft.notes.richtext.scheme.Document
import com.microsoft.notes.richtext.scheme.Media
import com.microsoft.notes.richtext.scheme.Paragraph
import com.microsoft.notes.store.Color
import com.microsoft.notes.store.Note
import com.microsoft.notes.uiThread
import com.microsoft.notes.utils.threading.ThreadExecutor


class NotesListControllerView(
        val uiFragmentComponent: UiNotesFragmentComponent,
        appStore: AppStore = com.microsoft.notes.appStore,
        mainThread: ThreadExecutor? = uiThread) : ControllerView(appStore, mainThread) {

    fun createAndNavigateToMediaNote(uri: String) {
        val document = Document(listOf(Media(localUrl = uri)))
        val newNote = Note(document = document)
        addAndNavigateToNote(newNote)
    }

    fun setNoteColor(note: Note, color: Color) {
        NotesLibrary.getInstance().updateNoteColor(note.localId, color)
    }

    fun reorderNotes(notesCollection: List<Note>) {
        NotesLibrary.getInstance().reorderNotes(notesCollection)
    }

    fun createAndNavigateToNote(paragraphText: String?) {
        val document = if (paragraphText != null) {
            Document(listOf(Paragraph(content = Content(text = paragraphText))))
        } else {
            Document()
        }
        val newNote = Note(document = document)
        addAndNavigateToNote(newNote)
    }

    fun navigatingToNote(note: Note) {
        val navigateToAction = NavigateToAction(Navigation.EDIT_NOTE)

        // TODO: remove remotedata from action
        val fetchNoteAction = FetchNoteDetailsAction(note.localId)

        appStore.dispatch(CompoundAction(fetchNoteAction, navigateToAction))
    }

    fun navigatingToSearch() {
        appStore.dispatch(NavigateToAction(Navigation.SEARCH))
    }

    fun navigatingToPreferences() {
        appStore.dispatch(NavigateToAction(Navigation.PREFERENCES))
    }

    private fun addAndNavigateToNote(newNote: Note) {
        NotesLibrary.getInstance().addNote(newNote).then {
            val fetchNoteDetailsAction = FetchNoteDetailsAction(newNote.localId)
            val navigateToAction = NavigateToAction(Navigation.EDIT_NOTE)
            appStore.dispatch(CompoundAction(fetchNoteDetailsAction, navigateToAction))
        }
    }

    override fun handleState(appState: AppState) {
        with(appState.notesList) {
            val notesFilter: (Note) -> Boolean = if (archivedNotesVisible) {
                { true and !(it.hasNoText and it.isMediaListEmpty)}
            } else {
                { !it.isArchived and !(it.hasNoText and it.isMediaListEmpty)}
            }
            uiFragmentComponent.updateNotesCollection(
                    notesCollection.filter(notesFilter), scrollTo)
        }
    }
}