package com.microsoft.notes.noteslist

import android.content.Context
import android.text.Editable
import android.text.TextWatcher
import android.util.AttributeSet
import android.view.View
import android.widget.EditText
import com.microsoft.notes.extensions.setTextWhenDifferent
import com.microsoft.notes.richtext.scheme.NEW_LINE_CHAR_AS_STR
import com.microsoft.notes.richtext.scheme.asString
import com.microsoft.notes.store.Note

interface TitleEditTextViewCallback {
    fun updateTitle(title: String)
    fun moveFocusToBody()
}

class TitleEditText(context: Context, attrs: AttributeSet?) : EditText(context, attrs) {
    private var titleEditTextViewCallback: TitleEditTextViewCallback? = null
    private var ignoreTextChangeCount = 0

    constructor(context: Context) : this(context, null)

    init {
        setupTextWatcher()
    }

    fun setTitleEditTextViewCallback(titleEditTextViewCallback: TitleEditTextViewCallback) {
        this.titleEditTextViewCallback = titleEditTextViewCallback
    }

    fun setTitleWhenDifferent(title: String) =
            setTextWhenDifferent(title)

    fun setTitleVisibilityWhenThereIsOnlyBodyText(note: Note) {
        if (note.title.isEmpty()
                && !note.document.asString().isEmpty()) {
            visibility = View.GONE
        }
    }

    private fun setupTextWatcher() =
            this.addTextChangedListener(object : TextWatcher {
                override fun afterTextChanged(editable: Editable) {
                    val newTitle = editable.toString()
                    if (newTitle.contains(NEW_LINE_CHAR_AS_STR)) {
                        setText(newTitle.replace(NEW_LINE_CHAR_AS_STR, ""))
                        titleEditTextViewCallback?.moveFocusToBody()
                    } else if (ignoreTextChangeCount == 0) {
                        notifyTitleIsUpdated(editable.toString())
                    }
                }

                override fun beforeTextChanged(text: CharSequence?, start: Int, count: Int, after: Int) = Unit

                override fun onTextChanged(text: CharSequence?, start: Int, before: Int, count: Int) = Unit
            })

    fun notifyTitleIsUpdated(title: String = text.toString()) {
        titleEditTextViewCallback?.updateTitle(title)
    }

    override fun setText(text: CharSequence?, type: BufferType?) {
        ignoreTextChangeCount++
        super.setText(text, type)
        ignoreTextChangeCount--
    }
}