package com.microsoft.notes.photos

import android.app.Activity
import android.content.pm.ResolveInfo
import android.support.v7.app.AlertDialog
import android.widget.GridView
import com.microsoft.notes.R
import java.util.ArrayList

enum class PhotoPickerType { CAMERA, GALLERY }
data class PhotoPickerItem(val intentResolveInfo: ResolveInfo, val type: PhotoPickerType)

class PhotoPickerDialog {

    fun showPictureLauncher(activity: Activity, handler: (PhotoPickerItem) -> Unit) {
        val context = activity.baseContext

        val takePictureIntent = createTakePhotoIntent()
        val pickPictureIntent = createPickPhotoIntent()

        val takePictureResults: MutableList<ResolveInfo>? =
                activity.packageManager.queryIntentActivities(takePictureIntent, 0)
        val pickPictureResults = activity.packageManager.queryIntentActivities(pickPictureIntent, 0)

        val options = ArrayList<PhotoPickerItem>()
        takePictureResults?.let { options.addAll(it.map { PhotoPickerItem(it, PhotoPickerType.CAMERA) }) }
        pickPictureResults?.let { options.addAll(it.map { PhotoPickerItem(it, PhotoPickerType.GALLERY) }) }

        val adapter = PhotoPickerAdapter(context, R.string.photo_picker_title, options)

        val builder = AlertDialog.Builder(activity)
        // Passing null is okay here because we are using this layout for an AlertDialog
        val gridView = activity.layoutInflater.inflate(R.layout.photo_picker_dialog_layout, null) as GridView

        builder.setView(gridView)
        gridView.adapter = adapter

        val dialog = builder.show()
        gridView.setOnItemClickListener { _, _, item, _ ->
            dialog.dismiss()
            handler(options[item])
        }
        dialog.ownerActivity = activity
    }
}