package com.microsoft.notes.preferences

import android.content.Context
import android.content.SharedPreferences
import android.preference.PreferenceManager
import com.microsoft.notes.appstore.AuthMethods
import com.microsoft.notes.appstore.PreferenceKeys.AUTH_METHOD
import com.microsoft.notes.appstore.SideEffect
import com.microsoft.notes.appstore.AppStore
import com.microsoft.notes.appstore.action.Action
import com.microsoft.notes.appstore.action.AuthAction.AADLoginAction
import com.microsoft.notes.appstore.action.AuthAction.LogOutAction
import com.microsoft.notes.appstore.action.CompoundAction
import com.microsoft.notes.appstore.action.ReadAction.PreferencesLoadedAction
import com.microsoft.notes.appstore.action.ReadAction.LoadPreferencesAction
import com.microsoft.notes.uiThread

class PreferencesSideEffect(val context: Context, val appStore: AppStore) : SideEffect(uiThread) {

    private val preferences: SharedPreferences
        get() = PreferenceManager.getDefaultSharedPreferences(context)

    private val allPrefKeys: List<String> by lazy {
        listOf(AUTH_METHOD)
    }

    override fun handle(data: Action) {
        when (data) {
            is CompoundAction -> data.actions.forEach { handle(it) }
            is LoadPreferencesAction -> loadPreferences()
            is AADLoginAction -> setPreference(AUTH_METHOD, AuthMethods.AAD)
            is LogOutAction -> setPreference(AUTH_METHOD, null)
        }
    }

    private fun loadPreferences() {
        val prefs = allPrefKeys.mapNotNull { key ->
            preferences.getString(key, null)?.let { Pair(key, it) }
        }.toMap()
        appStore.dispatch(PreferencesLoadedAction(prefs))
    }

    private fun setPreference(key: String, value: String?) {
        when (value) {
            null -> preferences.edit().remove(key).apply()
            else -> preferences.edit().putString(key, value).apply()
        }
    }
}