package com.microsoft.notes.search

import com.microsoft.notes.appstore.AppState
import com.microsoft.notes.appstore.AppStore
import com.microsoft.notes.appstore.Navigation
import com.microsoft.notes.appstore.ScrollTo
import com.microsoft.notes.appstore.action.CompoundAction
import com.microsoft.notes.appstore.action.NavigationAction
import com.microsoft.notes.appstore.action.NoteUIAction
import com.microsoft.notes.components.UiNotesFragmentComponent
import com.microsoft.notes.controllerview.ControllerView
import com.microsoft.notes.noteslib.NotesLibrary
import com.microsoft.notes.store.Color
import com.microsoft.notes.store.Note
import com.microsoft.notes.uiThread
import com.microsoft.notes.utils.threading.ThreadExecutor

interface Search {
    fun searchAsync(asyncThreadBlock: () -> List<Note>, uiThreadBlock: (List<Note>, ScrollTo) -> Unit)
}

class SearchControllerView(
        val uiFragmentComponent: UiNotesFragmentComponent,
        appStore: AppStore = com.microsoft.notes.appStore,
        mainThread: ThreadExecutor? = uiThread,
        private val search: Search) : ControllerView(appStore, mainThread) {

    var query: String = ""
        set(value) {
            field = value
            runSearch()
        }

    private val notesCollection: List<Note>
        get() {
            val notesFilter: (Note) -> Boolean
            if (appState.notesList.archivedNotesVisible) {
                notesFilter = { true }
            } else {
                notesFilter = { !it.isArchived }
            }

            return appState.notesList.notesCollection.filter(notesFilter)
        }

    fun setNoteColor(note: Note, color: Color) {
        NotesLibrary.getInstance().updateNoteColor(note.localId, color)
    }

    fun navigatingToNote(note: Note) {
        val navigateToAction = NavigationAction.NavigateToAction(Navigation.EDIT_NOTE)
        val fetchNoteAction = NoteUIAction.FetchNoteDetailsAction(note.localId)
        appStore.dispatch(CompoundAction(fetchNoteAction, navigateToAction))
    }


    private fun runSearch() {
        when {
            query.isEmpty() -> uiFragmentComponent.updateNotesCollection(emptyList(), ScrollTo.Top)
            else -> {
                search.searchAsync(
                        asyncThreadBlock = {
                            notesCollection.dumbTextSearch(query)
                        },
                        uiThreadBlock = { queryResult, scrollTo ->
                            if (isActivityRunning)
                                uiFragmentComponent.updateNotesCollection(queryResult, scrollTo)
                        })
            }
        }
    }

    override fun handleState(appState: AppState) {
        runSearch()
    }

}