package com.microsoft.notes.transition

import android.content.Context
import android.graphics.Color
import android.graphics.Rect
import android.graphics.drawable.GradientDrawable
import android.util.AttributeSet
import android.widget.FrameLayout

class AnimatableRoundedRect(context: Context, attrs: AttributeSet?) : FrameLayout(context, attrs) {

    constructor(context: Context) : this(context, null)

    var cornerRadius: Float = 0f
        set(value) {
            field = value
            updateBackground()
        }

    var color: Int = Color.BLACK
        set(value) {
            field = value
            updateBackground()
        }

    var epicenterX: Int = 0
        set(value) {
            field = value
            updatePosition()
        }

    var epicenterY: Int = 0
        set(value) {
            field = value
            updatePosition()
        }

    var backgroundOffsets: Rect = Rect(0, 0, 0, 0)
        set(value) {
            field = value
            updatePosition()
        }

    private fun updateBackground() {
        val drawable = GradientDrawable()
        drawable.setColor(color)
        drawable.cornerRadius = cornerRadius
        background = drawable
    }

    private fun updatePosition() {
        val layoutParams = FrameLayout.LayoutParams(this.layoutParams)
        layoutParams.leftMargin = epicenterX + backgroundOffsets.left
        layoutParams.topMargin = epicenterY + backgroundOffsets.top
        layoutParams.width = backgroundOffsets.width()
        layoutParams.height = backgroundOffsets.height()
        this.layoutParams = layoutParams
    }
}