package com.microsoft.notes.transition

import android.animation.Animator
import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.support.v7.widget.CardView
import android.transition.Transition
import android.transition.TransitionValues
import android.view.ViewGroup

class CardElevationTransition(val params: DynamicTransitionParams.Duration) : Transition() {
    companion object {
        val PROPNAME_ELEVATION = "com.microsoft.notes:CardElevationTransition:elevation"
    }

    override fun getDuration(): Long = params.duration

    override fun captureStartValues(transitionValues: TransitionValues?) {
        captureValues(transitionValues)
    }

    override fun captureEndValues(transitionValues: TransitionValues?) {
        captureValues(transitionValues)
    }

    override fun createAnimator(sceneRoot: ViewGroup?, startValues: TransitionValues?,
                                endValues: TransitionValues?): Animator? {
        if (startValues == null || endValues == null)
            return null

        val cardView = endValues.view as CardView
        val elevationAnimator = ObjectAnimator.ofFloat(endValues.view, "cardElevation",
                startValues.values[PROPNAME_ELEVATION] as Float, endValues.values[PROPNAME_ELEVATION] as Float)
        cardView.cardElevation = startValues.values[PROPNAME_ELEVATION] as Float

        val animator = AnimatorSet()
        animator.playTogether(elevationAnimator)
        return animator
    }

    private fun captureValues(transitionValues: TransitionValues?) {
        if (transitionValues == null) return
        val view = transitionValues.view as CardView
        transitionValues.values.put(PROPNAME_ELEVATION, view.cardElevation)
    }
}