package com.microsoft.notes.transition

import android.content.Context
import android.os.Build
import android.support.annotation.RequiresApi
import android.transition.Transition
import android.transition.TransitionSet
import android.view.Gravity
import com.microsoft.notes.R
import com.microsoft.notes.extensions.accelerateOrDecelerate
import com.microsoft.notes.extensions.addTransitions
import com.microsoft.notes.extensions.withSidePropagation
import com.microsoft.notes.noteslist.recyclerview.NoteItemComponent
import com.microsoft.notes.search.SquareView

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
object SearchTransitions {

    const val BUBBLES_FADE_DURATION = 100L
    const val RESULTS_FADE_DURATION = 150L
    fun bubblesFadeDelay(entering: Boolean) = if (entering) 75L else 0L
    fun resultsFadeDelay(entering: Boolean) = if (entering) 75L else 0L

    fun colorBubblesToResults(context: Context): Transition {
        return TransitionSet().addTransitions(
                animateColorBubbles(context, false),
                animateNoteItems(context, true)
        )
    }

    fun resultsToColorBubbles(context: Context): Transition {
        return TransitionSet().addTransitions(
                animateNoteItems(context, false),
                animateColorBubbles(context, true)
        )
    }

    private fun animateColorBubbles(context: Context, entering: Boolean) =
            shrinkInPlace().apply {
                addTarget(SquareView::class.java)
                duration = BUBBLES_FADE_DURATION
                startDelay = bubblesFadeDelay(true)
                accelerateOrDecelerate(!entering)
            }

    private fun animateNoteItems(context: Context, entering: Boolean) =
            slideAndFade(context, Gravity.BOTTOM).apply {
                addTarget(NoteItemComponent::class.java)
                duration = RESULTS_FADE_DURATION
                startDelay = resultsFadeDelay(entering)
                if (entering)
                    withSidePropagation(Gravity.BOTTOM, .75f)
            }

    private fun slideAndFade(context: Context, direction: Int): Transition {
        val dy = when (direction) {
            Gravity.TOP -> -context.resources.getDimension(R.dimen.color_filter_translate_animation)
            Gravity.BOTTOM -> context.resources.getDimension(R.dimen.color_filter_translate_animation)
            else -> 0f
        }
        return ScaleTranslateAndFadeTransition(dy = dy)
    }

    private fun shrinkInPlace() =
            ScaleTranslateAndFadeTransition(scale = .85f)
}