package com.microsoft.notes.transition

import android.os.Build
import android.support.annotation.RequiresApi
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentActivity
import android.support.v4.app.FragmentTransaction
import android.view.View
import com.microsoft.notes.R
import com.microsoft.notes.editnote.EditNoteFragment
import com.microsoft.notes.noteslist.NotesListComponent
import com.microsoft.notes.noteslist.NotesListFragment
import com.microsoft.notes.noteslist.recyclerview.NoteItemComponent
import com.microsoft.notes.search.SearchFragment
import com.microsoft.notes.store.Note
import kotlinx.android.synthetic.main.notes_layout.*

val transitionCoordinator: TransitionCoordinator by lazy {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
        TransitionCoordinatorApi21()
    else
        TransitionCoordinator()
}

open class TransitionCoordinator {
    var animationOverlayViewID: Int = View.NO_ID
	
    open fun default(activity: FragmentActivity, from: Fragment?, to: Fragment?, txn: FragmentTransaction) {
        txn.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE)
    }

    open fun notesListToEditNote(activity: FragmentActivity, from: NotesListFragment, to: EditNoteFragment,
                                 txn: FragmentTransaction, note: Note?)
            = default(activity, from as Fragment, to as Fragment, txn)

    open fun editNoteToNotesList(activity: FragmentActivity, from: EditNoteFragment, to: NotesListFragment,
                                 txn: FragmentTransaction)
            = default(activity, from as Fragment, to as Fragment, txn)

    open fun notesListToSearch(activity: FragmentActivity, from: NotesListFragment, to: SearchFragment,
                               txn: FragmentTransaction)
            = default(activity, from as Fragment, to as Fragment, txn)

    open fun searchToNotesList(activity: FragmentActivity, from: SearchFragment, to: NotesListFragment,
                               txn: FragmentTransaction)
            = default(activity, from as Fragment, to as Fragment, txn)

    open fun searchToEditNote(activity: FragmentActivity, from: SearchFragment, to: EditNoteFragment,
                              txn: FragmentTransaction, note: Note?)
            = default(activity, from as Fragment, to as Fragment, txn)

    open fun editNoteToSearch(activity: FragmentActivity, from: EditNoteFragment, to: SearchFragment,
                              txn: FragmentTransaction)
            = default(activity, from as Fragment, to as Fragment, txn)
}

@RequiresApi(Build.VERSION_CODES.LOLLIPOP)
class TransitionCoordinatorApi21 : TransitionCoordinator() {
    override fun default(activity: FragmentActivity, from: Fragment?, to: Fragment?, txn: FragmentTransaction) {
        super.default(activity, from, to, txn)
        from?.enterTransition = null
        from?.exitTransition = null
        to?.enterTransition = null
        to?.exitTransition = null
        to?.sharedElementEnterTransition = null
    }

    override fun notesListToEditNote(activity: FragmentActivity, from: NotesListFragment, to: EditNoteFragment,
                                     txn: FragmentTransaction, note: Note?) {
        //FIXME Much better to send as parameter the "origin from" instead checking for "note content"
        var noteView: NoteItemComponent? = null
        if (note != null && !note.isEmpty) {
            noteView = from.notesList.prepareForEditNoteTransition(note, txn)
        }

        if (noteView != null) {
            NotesListEditNoteTransitions.params.setFromViews(noteView, from.notesList)
        } else {
            from.notesList.prepareForNewNoteTransition()
            NotesListEditNoteTransitions.params.setFromViews(from.newNoteFab, from.notesList)
        }

        NotesListEditNoteTransitions.applyToEditNote(activity, from, to, noteView == null, activity.findViewById(animationOverlayViewID))
    }

    override fun editNoteToNotesList(activity: FragmentActivity, from: EditNoteFragment, to: NotesListFragment,
                                     txn: FragmentTransaction) {
        val isEmpty = from.isEmpty()
        to.view?.findViewById<NotesListComponent>(R.id.notesList)?.prepareForReturnTransition()

        if (!isEmpty) {
            from.prepareSharedElements { view, name ->
                view.transitionName = name
                txn.addSharedElement(view, name)
            }
        }

        NotesListEditNoteTransitions.applyFromEditNote(activity, from, to, isEmpty)

        to.addFirstLayoutListener {
            val noteView: NoteItemComponent? = if (!isEmpty) to.findCurrentNoteView()?.noteView else null
            if (noteView != null) {
                NotesListEditNoteTransitions.params.setFromViews(noteView, to.notesList)
            } else {
                NotesListEditNoteTransitions.params.setFromViews(to.newNoteFab, to.notesList)
            }
        }
    }

    override fun notesListToSearch(activity: FragmentActivity, from: NotesListFragment, to: SearchFragment,
                                   txn: FragmentTransaction) {
        NotesListToSearchTransitions.apply(activity, from, to)
    }

    override fun searchToNotesList(activity: FragmentActivity, from: SearchFragment, to: NotesListFragment,
                                   txn: FragmentTransaction) {
        NotesListToSearchTransitions.apply(activity, from, to, activity.findViewById(animationOverlayViewID))
    }

    override fun searchToEditNote(activity: FragmentActivity, from: SearchFragment, to: EditNoteFragment,
                                  txn: FragmentTransaction, note: Note?) {
        val notesList = from.notesList
        if (note == null || notesList == null) {
            default(activity, from, to, txn)
            return
        }

        val noteView = notesList.prepareForEditNoteTransition(note, txn)
        noteView?.let { NotesListEditNoteTransitions.params.setFromViews(noteView, notesList) }
        NotesListEditNoteTransitions.applyToEditNote(activity, from, to, isNewNote = false,
                animationOverlay = activity.findViewById(animationOverlayViewID))
    }

    override fun editNoteToSearch(activity: FragmentActivity, from: EditNoteFragment, to: SearchFragment,
                                  txn: FragmentTransaction) {
        val isEmpty = from.isEmpty()
        to.view?.findViewById<NotesListComponent>(R.id.notesList)?.prepareForReturnTransition()

        if (!isEmpty) {
            from.prepareSharedElements { view, name ->
                view.transitionName = name
                txn.addSharedElement(view, name)
            }
        }

        NotesListEditNoteTransitions.applyFromEditNote(activity, from, to, isEmpty)

        to.addFirstLayoutListener {
            to.notesList?.let {
                val noteView: NoteItemComponent? = if (!isEmpty) to.findCurrentNoteView()?.noteView else null
                if (noteView != null) {
                    NotesListEditNoteTransitions.params.setFromViews(noteView, it)
                } else {
                    NotesListEditNoteTransitions.params.setFromViews(to.newNoteFab, it)
                }
            }
        }
    }
}
